package com.zulutrade.sample.trading.client.trading;

import com.zulutrade.sample.trading.client.trading.dtos.ClosedTradesResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.NewMarketTradeResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.NewOrderResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.OpenTradesResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.OrderCanceledResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.ProviderStatisticsResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.TradeClosedResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.TradingConfigurationResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.UpdateTrailingStopValueResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.UpdateValueResponseMessage;

/**
 * Available trading methods
 * exposed by zulutrade, in case of an exception, exception's
 * message will hold server's response if available
 *
 * @author ZuluTrade
 */
/**
 * 
 * @author ZuluTrade
 */
public interface ZuluTradingGateway {

	/**
	 * Opens a market trade at current market rate, new trade will be assigned the uniqueId specified in this call sou you are able to
	 * modify/close in the future
	 * 
	 * @param currencyName The instrument name to be traded
	 * @param lots Volume in standard lots
	 * @param buy BUY/SELL
	 * @param requestedPrice indicative price, ZuluTrade will open at current market rate
	 * @param uniqueId Id of the trading request, in case call is successful, you'll be able to close/modify the trade using this id
	 * @return {@link NewMarketTradeResponseMessage}
	 * @throws Exception
	 */
	NewMarketTradeResponseMessage openMarket(String currencyName, double lots, boolean buy, double requestedPrice, String uniqueId)
			throws Exception;

	/**
	 * Opens a pending order, new pending order will be assigned the uniqueId specified in this call sou you are able to modify/close in the
	 * future
	 * @param currencyName The instrument name to be traded
	 * @param lots Volume in standard lots
	 * @param buy BUY/SELL
	 * @param execution price
	 * @param uniqueId Id of the trading request, in case call is successful, you'll be able to close/modify the trade using this id
	 * @return {@link NewMarketTradeResponseMessage}
	 * @throws Exception
	 */
	NewOrderResponseMessage openOrder(String currencyName, double lots, boolean buy, double requestedPrice, String uniqueId)
			throws Exception;

	/**
	 * Cancels a pending order with the provided uniqueId
	 * 
	 * @param currencyName The instrument name to be traded
	 * @param lots Volume in standard lots
	 * @param buy BUY/SELL
	 * @param uniqueId the id provided in relevant openOrder request
	 * @return {@link OrderCanceledResponseMessage}
	 * @throws Exception
	 */
	OrderCanceledResponseMessage cancelOrder(String currencyName, double lots, boolean buy, String uniqueId) throws Exception;

	/**
	 * Closes a market trade with the provided uniqueId
	 * @param currencyName The instrument name to be traded
	 * @param lots Volume in standard lots
	 * @param buy BUY/SELL
	 * @param uniqueId the id provided in relevant openOrder/openMarket request
	 * @param requestedPrice indicative request price, Zulutrade will execute at current market rate
	 * @return {@link TradeClosedResponseMessage}
	 * @throws Exception
	 */
	TradeClosedResponseMessage closeMarket(String currencyName, double lots, boolean buy, String uniqueId, double requestedPrice)
			throws Exception;

	/**
	 * Updates the stop value of a pending order/market trade
	 * @param currencyName The instrument name to be traded
	 * @param buy BUY/SELL
	 * @param lots lots Volume in standard lots
	 * @param uniqueId uniqueId the id provided in relevant openOrder/openMarket request
	 * @param stopValue the new stop value
	 * @return {@link UpdateValueResponseMessage}
	 * @throws Exception
	 */
	UpdateValueResponseMessage updateStop(String currencyName, boolean buy, double lots, String uniqueId, double stopValue)
			throws Exception;

	/**
	 * Updates the trailing stop value of a pending order/market trade
	 * @param currencyName The instrument name to be traded
	 * @param buy BUY/SELL
	 * @param lots lots Volume in standard lots
	 * @param uniqueId uniqueId the id provided in relevant openOrder/openMarket request
	 * @param tralingStopValue the tralining stop value
	 * @return {@link UpdateTrailingStopValueResponseMessage}
	 * @throws Exception
	 */
	UpdateTrailingStopValueResponseMessage updateTrailingStop(String currencyName, boolean buy, double lots, String uniqueId,
			int trailingStopValue) throws Exception;
	
	/**
	 * Updates the traling stop value of a pending order/market trade when profit reaches a certain profit in pips
	 * @param currencyName The instrument name to be traded
	 * @param buy BUY/SELL
	 * @param lots lots Volume in standard lots
	 * @param uniqueId uniqueId the id provided in relevant openOrder/openMarket request
	 * @param tralingStopValue the trailing stop value
	 * @param tralingStopConditionPipsToSet the profit in pips that when reached it will set the trailing stop
	 * @return {@link UpdateTrailingStopValueResponseMessage}
	 * @throws Exception
	 */
	UpdateTrailingStopValueResponseMessage updateTrailingStopCondition(String currencyName, boolean buy, double lots, String uniqueId,
			int trailingStopValueToSet, int tralingStopConditionPipsToSet) throws Exception;

	/**
	 * Updates the limit value of a pending order/market trade
	 * @param currencyName
	 * @param buy
	 * @param lots
	 * @param uniqueId uniqueId the id provided in relevant openOrder/openMarket request
	 * @param limitValue the new limit value
	 * @return {@link UpdateValueResponseMessage}
	 * @throws Exception
	 */
	UpdateValueResponseMessage updateLimit(String currencyName, boolean buy, double lots, String uniqueId, double limitValue)
			throws Exception;

	/**
	 * Updates the entry of a pending order
	 * @param currencyName
	 * @param buy
	 * @param lots
	 * @param uniqueId uniqueId the id provided in relevant openOrder/openMarket request
	 * @param entryValue the new entry rate
	 * @return {@link UpdateValueResponseMessage}
	 * @throws Exception
	 */
	UpdateValueResponseMessage updateEntryRate(String currencyName, boolean buy, double lots, String uniqueId, double entryValue)
			throws Exception;

	/**
	 * Returns the open market trades and pending orders in your account
	 * 
	 * @return {@link OpenTradesResponseMessage}
	 */
	OpenTradesResponseMessage getOpenTrades() throws Exception;

	/**
	 * Returns the minimum amount and minimum amount step available for opening market trades and pending orders
	 * 
	 * @return {@link TradingConfigurationResponseMessage}
	 */
	TradingConfigurationResponseMessage getTradingConfiguration() throws Exception;

	/**
	 * Returns the provider's performance page statistics, if the account is a provider 
	 * 
	 * @return {@link ProviderStatisticsResponseMessage}
	 */
	ProviderStatisticsResponseMessage getProviderStatistics() throws Exception;

	/**
	 * Returns the account's historic trades/orders that got closed between the given startDate and endDate
	 *  
	 * @param startDate StartDate in format yyyy-MM-dd
	 * @param endDate endDate in format yyyy-MM-dd
	 * @return {@link ClosedTradesResponseMessage}
	 */
	ClosedTradesResponseMessage getHistory(String startDate, String endDate) throws Exception;

}
